use axum::{
    extract::{Path, Request, State},
    middleware::{self, Next},
    response::{IntoResponse, Response},
    routing::get,
    Json, Router,
};
use http::{HeaderMap, HeaderValue, StatusCode};
use rand::distributions::{Alphanumeric, DistString};
use sqlx::{Pool, Postgres, QueryBuilder};

use crate::{markup_response::simple_error_page, model::Chat, state::AppState, ADMIN_TOK};

pub fn router(state: AppState) -> Router {
    Router::new()
        .route("/new/:amount", get(create))
        .nest("/stat", stat::router(state.clone()))
        .route_layer(middleware::from_fn_with_state(
            state.clone(),
            auth_middleware,
        ))
        .with_state(state)
}

async fn auth_middleware(
    State(state): State<AppState>,
    headers: HeaderMap,
    req: Request,
    next: Next,
) -> Response {
    let admin_tok = headers.get("x-admin-tok");
    if headers.get("x-admin-tok") == Some(&HeaderValue::from_static(ADMIN_TOK)) {
        let res = next.run(req).await;
        return res;
    }

    simple_error_page(StatusCode::UNAUTHORIZED).into_response()
}

mod stat {
    use axum::{extract::State, routing::get, Json, Router};

    use crate::{model::Chat, state::AppState};

    // TODO: /stat/* should require authentication
    pub fn router(state: AppState) -> Router<AppState> {
        Router::new().route("/chats", get(chats)).with_state(state)
    }

    async fn chats(State(state): State<AppState>) -> Json<Vec<Chat>> {
        let r = sqlx::query_as!(Chat, "select * from chats;")
            .fetch_all(state.pool())
            .await
            .unwrap();

        Json(r)
    }
}

async fn create(Path(amount): Path<u8>, State(state): State<AppState>) -> Json<Vec<Chat>> {
    let paths: Vec<String> = (0..amount)
        .map(|_| Alphanumeric.sample_string(&mut rand::thread_rng(), 6))
        .collect();

    let r: Vec<Chat> = QueryBuilder::new("insert into chats (url_path)")
        .push_values(paths, |mut b, v| {
            b.push_bind(v);
        })
        .push("returning *")
        .build_query_as()
        .fetch_all(state.pool())
        .await
        .unwrap();

    Json(r)
}
